from enum import Enum


class GCMAlgorithmNames(Enum):
    GCM_TYPE = "GCM_type"
    MOTIF_SIZES = "motif_sizes"
    BUILD_FUNCTIONS = "build_functions"
    EDGE_NAMES = "edge_names"
    MOTIF_INDICES = "motif_indices"
    MOTIF_ID = "motif_id"
