from enum import Enum


class ToolsNames(Enum):
    NETWORK = "network"
    JDD = "jdd"
    EDGE_NAMES = "edge_names"
    EJKS = "ejks"
    EXCESS_DEGREE_KEYS = "excess_degree_keys"
    CONVERGENCE_LIMIT = "convergence_limit"
    SEARCH_LIMIT = "search_limit"
