from enum import Enum


class JointDegreeNames(Enum):
    JOINT_DEGREE_TYPE = "joint_degree_type"
    COVER = "cover"
    TARGET_K = "target_k"
    FP = "fp"
    PROBS = "probs"
    MOTIF_SIZES = "motif_sizes"
    LOW_HIGH_DEGREE_BOUND = "low_high_degree_bound"
    JDS = "jds"
    JDD = "jdd"
    ARR_FP = "arr_fp"
    USE_SAMPLING = "use_sampling"
    N_SAMPLES = "n_samples"
